@@ -1,6 +1,6 @@
 
-#include <config.h>
-#include <mypaint-config.h>
+#include "config.h"
+#include "mypaint-config.h"
 
 #ifdef _OPENMP
 #include <omp.h>
